<?php

namespace FiloBlu\EsbRma\Model\Rest;

use Exception;
use FiloBlu\Esb\Api\Data\AcknowledgeInterface;
use FiloBlu\Esb\Helper\LoggerProvider;
use FiloBlu\EsbRma\Api\ExternalRmaInterface;
use FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filesystem;
use Magento\Rma\Api\Data\RmaInterface;
use Magento\Rma\Api\RmaManagementInterface;
use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Rma\Model\Rma\Status\HistoryFactory;
use Magento\Sales\Api\Data\OrderExtensionFactory;
use Magento\Sales\Exception\CouldNotRefundException;
use Magento\Sales\Model\OrderRepository;
use Monolog\Logger;

/**
 * REST Api to save external rma
 */
class ExternalRma implements ExternalRmaInterface
{

    /**
     * @var OrderExtensionFactory
     */
    protected $_orderRepository;

    /**
     * @var SearchCriteriaBuilder
     */
    protected $_searchCriteriaBuilder;

    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var OrderExtensionFactory
     */
    protected $_orderExtensionFactory;

    /**
     * @var RmaRepository
     */
    protected $_rmaRepository;

    /**
     * @var RmaManagement
     */
    protected $_rmaManagement;

    /**
     * @var commentManagement
     */
    protected $_historyFactory;

    /**
     * @var Filesystem
     */
    protected $_filesystem;

    /**
     * @var Logger
     */
    protected $_logger;

    /**
     * @var AcknowledgeInterface
     */
    protected $_response;

    /**
     * @var AdditionalDataFactory
     */
    protected $_additionalDataFactory;

    /**
     * Init plugin
     *
     * @param ScopeConfigInterface $scopeConfig
     * @param OrderRepository $orderRepository
     * @param OrderExtensionFactory $orderExtensionFactory
     * @param RmaRepositoryInterface $rmaRepository
     * @param RmaManagementInterface $rmaManagement
     * @param \Magento\Rma\Model\Rma\Status\HistoryFactory $historyFactory
     * @param Filesystem $filesystem
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param LoggerProvider $loggerProvider
     * @param AcknowledgeInterface $response
     * @param AdditionalDataFactory $additionalDataFactory
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        OrderRepository $orderRepository,
        OrderExtensionFactory $orderExtensionFactory,
        RmaRepositoryInterface $rmaRepository,
        RmaManagementInterface $rmaManagement,
        HistoryFactory $historyFactory,
        Filesystem $filesystem,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        LoggerProvider $loggerProvider,
        AcknowledgeInterface $response,
        AdditionalDataFactory $additionalDataFactory
    ) {
        $this->_scopeConfig = $scopeConfig;
        $this->_orderRepository = $orderRepository;
        $this->_orderExtensionFactory = $orderExtensionFactory;
        $this->_rmaRepository = $rmaRepository;
        $this->_rmaManagement = $rmaManagement;
        $this->_historyFactory = $historyFactory;
        $this->_filesystem = $filesystem;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_logger = $loggerProvider->getLogger();
        $this->_response = $response;
        $this->_additionalDataFactory = $additionalDataFactory;
    }

    /**
     * Save External Creditmemo data. Save PDF file and additional data on Sales Order.
     *
     * @param RmaInterface $rma_data
     * @return AcknowledgeInterface
     * @throws CouldNotRefundException
     * @throws FileSystemException
     * @throws LocalizedException
     * @throws Exception
     */
    public function update(RmaInterface $rma_data)
    {
        return $this->saveRmadata($rma_data);
    }

    /**
     * @param $rma_data
     * @param $rma_status
     * @param $item_status
     * @param $notify
     * @return \FiloBlu\Esb\Api\Data\AcknowledgeInterface
     */
    protected function saveRmaData($rma_data, $rma_status = null, $item_status = null, $notify = false)
    {
        try {
            $searchCriteria = $this->_searchCriteriaBuilder
                ->addFilter('increment_id', $rma_data->getIncrementId())
                ->create();

            $orderList = $this->_rmaRepository->getList($searchCriteria);
            $items = $orderList->getItems();
            $rma = array_shift($items);

            $rma = $this->_rmaRepository->get($rma->getId());

            $rma_data->setEntityId($rma->getEntityId());
            $rma_data->setOrderId($rma->getOrderId());
            $rma_data->setOrderIncrementId($rma->getOrderIncrementId());
            $rma_data_items = $rma_data->getItems();

            if ($item_status === 'approved') {
                foreach ($rma_data_items as $key => $input_item) {
                    $input_item->setQtyApproved($input_item->getQtyReturned());
                    $rma_data_items[$key] = $input_item;
                }
            }
            foreach ($rma->getItems() as $rma_item) {
                foreach ($rma_data_items as $key => $input_item) {


                    if ($input_item->getEntityId()) {
                        if ($input_item->getEntityId() == $rma_item->getEntityId()) {
                            $this->setResolution($input_item, $rma_item);
                            $this->_logger->info('Item found by ID: ' . $input_item->getEntityId());
                            $input_item->setRmaEntityId($rma_item->getRmaEntityId());
                            $input_item->setOrderItemId($rma_item->getOrderItemId());
                            if ($item_status != null) {
                                $input_item->setStatus($item_status);
                            }
                            $rma_data_items[$key] = $input_item;
                            break;
                        }
                    } elseif ($input_item->getOrderItemId()) {
                        if ($input_item->getOrderItemId() == $rma_item->getOrderItemId()) {
                            $this->setResolution($input_item, $rma_item);
                            $this->_logger->info('Item found by Order Item ID: ' . $input_item->getOrderItemId());
                            $input_item->setEntityId($rma_item->getEntityId());
                            $input_item->setRmaEntityId($rma_item->getRmaEntityId());
                            if ($item_status != null) {
                                $input_item->setStatus($item_status);
                            }
                            $rma_data_items[$key] = $input_item;
                            break;
                        }
                    } else {
                        $inputItemExtensionAttributes = $input_item->getExtensionAttributes();
                        $rmaItemExtensionAttributes = $rma_item->getExtensionAttributes();
                        if ($inputItemExtensionAttributes->getSku() === $rmaItemExtensionAttributes->getSku()) {
                            $this->setResolution($input_item, $rma_item);
                            $this->_logger->info('Item found by Sku: ' . $inputItemExtensionAttributes->getSku());
                            $input_item->setEntityId($rma_item->getEntityId());
                            $input_item->setRmaEntityId($rma_item->getRmaEntityId());
                            $input_item->setOrderItemId($rma_item->getOrderItemId());
                            if ($item_status != null) {
                                $input_item->setStatus($item_status);
                            }

                            $rma_data_items[$key] = $input_item;
                            break;
                        }
                    }
                }
            }
            $rma_data->setItems($rma_data_items);
            $comment = __('Your Return request has been saved');
            if ($rma_status != null) {
                $rma_data->setStatus($rma_status);

                /* Taken from module-rma/Controller/Adminhtml/Rma/AddComment.php*/
                $history = $this->_historyFactory->create();
                $history->setRmaEntityId($rma->getId());
                $comment = __('Your Return request has been %1', $item_status);
                $history->setComment($comment);
                if ($notify) {
                    $history->sendCommentEmail();
                }
                $history->saveComment($comment, $notify, true);
            }
            $this->_rmaManagement->saveRma($rma_data);

            $this->_response->setStatus('success');
            $this->_response->setOrderIncrementId($rma->getOrderIncrementId());
            $this->_response->setMessage($comment);
            return $this->_response;
        } catch (\Exception $e) {
            $this->_logger->critical($e->getMessage());
            $this->_response->setStatus('error');
            $this->_response->setMessage(__($e->getMessage()));
            $this->_response->setDescription(__($e->getTraceAsString()));
            return $this->_response;
        }
    }

    /**
     * Save External Creditmemo data. Save PDF file and additional data on Sales Order.
     *
     * @param RmaInterface $rma_data
     * @return AcknowledgeInterface
     * @throws CouldNotRefundException
     * @throws FileSystemException
     * @throws LocalizedException
     * @throws Exception
     */
    public function receive(RmaInterface $rma_data)
    {
        return $this->saveRmadata($rma_data, 'received');
    }

    /**
     * Save External Creditmemo data. Save PDF file and additional data on Sales Order.
     *
     * @param RmaInterface $rma_data
     * @return AcknowledgeInterface
     * @throws CouldNotRefundException
     * @throws FileSystemException
     * @throws LocalizedException
     * @throws Exception
     */
    public function receiveAndApprove(RmaInterface $rma_data)
    {
        $received = $this->saveRmadata($rma_data, 'received', 'received', true);
        if ($received->getStatus() !== 'error') {
            return $this->saveRmadata($rma_data, 'processed_closed', 'approved', false);
        }

        return $received;
    }

    /**
     * @param \Magento\Rma\Api\Data\RmaInterface $rma_data
     * @return \FiloBlu\Esb\Api\Data\AcknowledgeInterface
     */
    public function receiveAndNotify(RmaInterface $rma_data)
    {
        return $this->saveRmadata($rma_data, 'received', "received", true);
    }

    /**
     * @param \Magento\Rma\Api\Data\RmaInterface $rma_data
     * @return \FiloBlu\Esb\Api\Data\AcknowledgeInterface
     */
    public function approveAndNotify(RmaInterface $rma_data)
    {
        return $this->saveRmadata($rma_data, null, 'approved', true);
    }

    public function setResolution($input_item, $rma_item)
    {
        if (!$input_item->getResolution()) {
            $input_item->setResolution($rma_item->getResolution());
        }
    }
}
